QMainWindow {
    background: rgb(49, 52, 59);
    color: rgb(255, 255, 255);
    border-color: rgb(188, 188, 188);
}

QWidget {
    font-size: 10pt;
}

QMainWindow::separator {
    background: rgb(49, 52, 59);
}

QMenu {
    background-color: rgb(49, 52, 59);
    color: rgb(255, 255, 255);
    border-color: rgb(188, 188, 188);
}

QMenu::item {
    background-color: rgb(49, 52, 59);
    color: white;
    padding: 5px 50px 5px 35px;
    margin-right: 2px;
}

QMenu::item:selected {
    background-color: lightskyblue;
    color: black;
    border-color: rgb(188, 188, 188);
}

QMenuBar {
    background: rgb(49, 52, 59);
    color: white;
}

QMenuBar::item:selected {
    background-color: lightskyblue;
    color: black;
}

QPushButton {
    background-color: rgb(49, 52, 59);
    color: rgb(255, 255, 255);
    border-radius: 5px;
    border: 1px solid rgb(188, 188, 188);
    padding: 4px 0 4px 0;
    margin: 0 5px 0 5px;
}

QPushButton:hover {
    background-color: rgb(59, 62, 69);
    color: rgb(255, 255, 255);
    border-radius: 5px;
    border: 1px solid rgb(188, 188, 188);
    padding: 4px 0 4px 0;
    margin: 0 5px 0 5px;
}

QHeaderView::section {
    background-color: lightskyblue;
    color: black;
    font: bold;
}

QTableWidget {
    background-color: rgb(49, 52, 59);
    color: rgb(255, 255, 255);
}

QTableView::item {
    background-color: rgb(49, 52, 59);
    color: white;
}

QTableView::item:alternate {
    background-color: rgb(69, 72, 79);
    color: white;
}

Qlabel {
    background-color: transparent;
    color: rgb(255, 255, 255);
}

QGroupBox {
    background-color: rgb(49, 52, 59);
    color: rgb(255, 255, 255);
    border-radius: 5px;
    border: 1px solid rgb(188, 188, 188);
    padding: 10px 0 10px 0;
    font: bold;
    font-size: 15px;
    margin: 10px 5px 10px 5px;
}

QGroupBox::title {
    subcontrol-origin: margin;
    left: 20px;
}

QRadioButton::title {
    color: white;
}

QLineEdit {
    background-color: rgb(39, 42, 49);
    color: rgb(255, 255, 255);
    border: 1px solid rgb(188, 188, 188);
    height: 20px;
    border-radius: 5px;
    padding-left: 5px;
}

QLineEdit:disabled {
    background-color: rgb(59, 62, 69);
    color: rgb(255, 255, 255);
    border: 1px solid rgb(188, 188, 188);
    height: 20px;
    border-radius: 5px;
    padding-left: 10px;
}

QMessageBox {
    background-color: rgb(49, 52, 59);
    color: rgb(255, 255, 255);
    border-color: rgb(188, 188, 188);
}

QSplitter::handle {
    background: rgb(49, 52, 59);
    image: url(assets/ui/light_split_handle.png);
    height: 15px;
    padding: 0 -5px 0 -5px;
    color: rgb(255, 255, 255);
}

QDialog {
    background-color: rgb(49, 52, 59);
    color: rgb(255, 255, 255);
    border-color: rgb(188, 188, 188);
}

QComboBox {
    background-color: rgb(39, 42, 49);
    color: rgb(255, 255, 255);
    border: 1px solid whitesmoke;
    border-radius: 5px;
    height: 20px;
}

QComboBox:hover {
    background-color: rgb(59, 62, 69);
    color: rgb(255, 255, 255);
    border: 1px solid whitesmoke;
    border-radius: 5px;
    height: 20px;
}

QComboBox::down-arrow {
    border: none;
    image: url(assets/ui/light_down_arrow.png);
    width: 10px;
    margin-right: 5px;
}

QComboBox::down-arrow:hover {
    border: none;
    image: url(assets/ui/light_down_arrow.png);
    width: 14px;
    margin-right: 5px;
}

QComboBox::drop-down {
    border: none;
}

QComboBox QAbstractItemView{
    background-color: rgb(49, 52, 59);
    color: rgb(255, 255, 255);
    border: 1px solid darkgray;
    border-radius: 5px;
    height: 20px;
}

QProgressBar {
    background-color: rgba(0, 0, 0, 0);
    border: 2px solid grey;
    border-radius: 5px;
    color: rgb(255, 255, 255);
    min-height: 10px;
    max-height: 15px;
}

QProgressBar::chunk {
    background-color: #05B8CC;
    min-width: 1px;
    max-width: 1px;
}

QScrollBar:vertical {
    background: transparent;
}

QScrollBar::handle:vertical {
    border: 1px solid whitesmoke;
    background: rgb(49, 52, 59);
    border-radius: 8px;
}

QScrollBar::add-line:vertical {
    background: none;
}

QScrollBar::sub-line:vertical {
    background: none;
}

QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {
    background: none;
}

QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {
    background: none;
}

QScrollBar:horizontal {
    background: transparent;
}

QScrollBar::handle:horizontal {
    border: 1px solid whitesmoke;
    background: rgb(49, 52, 59);
    border-radius: 8px;
}

QScrollBar::add-line:horizontal {
    background: none;
}

QScrollBar::sub-line:horizontal {
    background: none;
}

QScrollBar::up-arrow:horizontal, QScrollBar::down-arrow:horizontal {
    background: none;
}

QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal {
    background: none;
}

QScrollArea {
    background-color: transparent;
}

QScrollArea QWidget{
    background-color: rgb(39, 42, 49);
    background-attachment: scroll;
    color: white;
}

QScrollArea QComboBox {
    background-color: rgb(49, 52, 59);
    color: rgb(255, 255, 255);
    border: 1px solid rgb(188, 188, 188);
    border-radius: 5px;
    height: 20px;
}

QScrollArea QComboBox QAbstractItemView{
    background-color: rgb(49, 52, 59);
    color: rgb(255, 255, 255);
    border: 1px solid darkgray;
    border-radius: 5px;
    height: 20px;
}

QScrollArea QComboBox QScrollBar:vertical{
    background: rgb(49, 52, 59);
}

QScrollArea QComboBox QScrollBar::handle:vertical {
    border: 1px solid whitesmoke;
    background: rgb(49, 52, 59);
    border-radius: 8px;
}

QScrollArea QComboBox QScrollBar::add-line:vertical {
    background: none;
}

QScrollArea QComboBox QScrollBar::sub-line:vertical {
    background: none;
}

QScrollArea QComboBox QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {
    background: none;
}

QScrollArea QComboBox QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {
    background: none;
}